/*
 * Copyright (C) 2008 ZXing authors
 * Copyright 2011 Robert Theis
 * Copyright 2012 Jaime Navarro Santapau
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file    CaptureActivityForOcr.java
 * @author  Jaime Navarro Santapau
 * @brief   Contiene los métodos y atributos de la clase CaptureActivity
 * @date    Última modificación Diciembre 2012
 * 
 */

/** 
 * @mainpage Pagina Principal - OCR Configurable
 *
 * @section intro_sec Introducción
 *
 * 	Esta aplicación realiza el procesado OCR de imagenes mediante la librería Tesseract. Para más información visite <a href="http://code.google.com/p/tesseract-ocr/">http://code.google.com/p/tesseract-ocr/</a>.
 *	<ul>
 *	 <li>Permite configurar ciertos parámetros para optimizar la adquisición de las imagenes:</li>
 *		<ul>
 *			<li>Autoenfoque.</li>
 *			<li>Luz delantera.</li>
 *		</ul>
 *	 <li>Permite configurar ciertos parámetros del motor Tesseract:</li>
 *	 	<ul>
 *			<li>Procesar las imágenes mediante un patrón específico realizado por nosotros, que debemos situar en "/mnt/sdcard/Android/data/edu.uoc.ocr/files/tessdata" con el nombre man.traineddata.</li>
 *			<li>Procesar las imágenes mediante alguno de los patrones generales descargados desde <a href="http://tesseract-ocr.googlecode.com/files/">http://tesseract-ocr.googlecode.com/files/</a>. </li>
 *			<li>Definir la lista de caracteres permitidos.</li>
 *			<li>Definir la lista de caracteres prohibidos.</li>
 *			<li>Seleccionar el modo de ejecución de la librería Tesseract.</li>
 *		</ul>
 *	</ul>
 *	@author	 Jaime Navarro Santapau
 *	@date      Diciembre 2012
 *	@version   1.0
 *	@copyright Licensed under the Apache License, Version 2.0
 */

package edu.uoc.ocr;

import com.googlecode.tesseract.android.TessBaseAPI;

import edu.uoc.ocr.R;
import edu.uoc.ocr.BeepManager;
import edu.uoc.ocr.HelpActivity;
import edu.uoc.ocr.OcrResult;
import edu.uoc.ocr.PreferencesActivity;
import edu.uoc.ocr.camera.CameraManager;
import edu.uoc.ocr.camera.ShutterButton;
import edu.uoc.ocr.LanguageCodeHelper;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.OnSharedPreferenceChangeListener;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Rect;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.preference.PreferenceManager;
import android.text.ClipboardManager;

import android.util.Log;
import android.util.TypedValue;
import android.view.ContextMenu;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.view.ContextMenu.ContextMenuInfo;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import java.io.File;
import java.io.IOException;
import java.util.Locale;

/**
 * @class CaptureActivityForOcr
 * @brief Esta actividad abre la cámara e inicia el procesado real en un
 *        subproceso de fondo. Se dibuja un Visor para ayudar al usuario a a
 *        limitar los limites de la imagen a procesar. Finalmente se superponen
 *        los resultados cuando finaliza el procesado de forma exitosa.
 */
public final class CaptureActivityForOcr extends Activity implements
		SurfaceHolder.Callback, ShutterButton.OnShutterButtonListener {

	private static final String TAG = CaptureActivityForOcr.class.getSimpleName();

	// Note: These constants will be overridden by any default values defined in
	// preferences.xml.

	/**
	 * @brief ISO 639-3 lenguaje de código que indica el idioma de reconocimiento
	 *        por defecto.
	 */
	public static String DEFAULT_SOURCE_LANGUAGE_CODE;
	
	static {
		Locale locale = Locale.getDefault();

		if (locale.getLanguage().contentEquals("es"))
			DEFAULT_SOURCE_LANGUAGE_CODE = "spa";
		else if (locale.getLanguage().contentEquals("en"))
			DEFAULT_SOURCE_LANGUAGE_CODE = "eng";
		else if (locale.getLanguage().contentEquals("ca"))
			DEFAULT_SOURCE_LANGUAGE_CODE = "cat";
		else
			DEFAULT_SOURCE_LANGUAGE_CODE = "spa";
	}
	
	
	  
	/** @brief Modo de funcionamiento de la libreria Tesseract. */
	public static final int OEM_TESSERACT_ONLY = 0;

	/** @brief El modo por defecto de segmentación a usar. */
	public static final String DEFAULT_PAGE_SEGMENTATION_MODE = "Auto";

	/** @brief Si se debe usar el enfoque automático por defecto. */
	public static final boolean DEFAULT_TOGGLE_AUTO_FOCUS = false;

	/**
	 * @brief Ya sea para que suene de forma predeterminada al pulsar el botón
	 *        del obturador.
	 */
	public static final boolean DEFAULT_TOGGLE_BEEP = true;

	/** @brief Ya sea que la luz debe ser inicialmente activado por defecto. */
	public static final boolean DEFAULT_TOGGLE_LIGHT = false;

	/**
	 * @brief Marcar para activar la visualización del botón de disparo en la
	 *        pantalla.
	 */
	private static final boolean DISPLAY_SHUTTER_BUTTON = true;

	/** @brief Recursos a utilizar para la descarga de archivos de datos. */
	static final String DOWNLOAD_BASE = "http://tesseract-ocr.googlecode.com/files/";

	/**
	 * @brief Descarga de nombre de archivo para detección de orientación y guión
	 *        (OSD) de datos.
	 */
	static final String OSD_FILENAME = "tesseract-ocr-3.01.osd.tar";

	/**
	 * @brief Destino nombre de archivo para la orientación y detección de
	 *        secuencias de comandos (OSD) de datos.
	 */
	static final String OSD_FILENAME_BASE = "osd.traineddata";

	/** @brief Patron de referencia generado manualmente. */
	static final String MANUAL_FILENAME_BASE = "man.traineddata";

	/**
	 * @brief Puntuación mínima confianza media necesaria para no rechazar un
	 *        solo disparo resultado del reconocimiento. Actualmente no se usa.
	 */
	static final int MINIMUM_MEAN_CONFIDENCE = 0; // 0 means don't reject any
													// scored results

	// Context menu
	private static final int SETTINGS_ID = Menu.FIRST;
	private static final int ABOUT_ID = Menu.FIRST + 1;

	// Options menu, for copy to clipboard
	private static final int OPTIONS_COPY_RECOGNIZED_TEXT_ID = Menu.FIRST;
	private static final int OPTIONS_COPY_TRANSLATED_TEXT_ID = Menu.FIRST + 1;
	private static final int OPTIONS_SHARE_RECOGNIZED_TEXT_ID = Menu.FIRST + 2;
	private static final int OPTIONS_SHARE_TRANSLATED_TEXT_ID = Menu.FIRST + 3;

	private CameraManager cameraManager;
	private CaptureActivityHandler handler;
	private ViewfinderView viewfinderView;
	private SurfaceView surfaceView;
	private SurfaceHolder surfaceHolder;
	private TextView statusViewBottom;
	private TextView statusViewTop;
	private TextView ocrResultView;
	private TextView translationView;
	private View cameraButtonView;
	private View resultView;
	private View progressView;
	private OcrResult lastResult;
	private Bitmap lastBitmap;
	private boolean hasSurface;
	private BeepManager beepManager;
	private TessBaseAPI baseApi; // Java interface for the Tesseract OCR engine
	private String sourceLanguageCodeOcr; // ISO 639-3 language code
	private String sourceLanguageReadable; // Language name, for example,
											// "English"
	private int pageSegmentationMode = TessBaseAPI.PageSegMode.PSM_AUTO;
	private int ocrEngineMode = OEM_TESSERACT_ONLY;
	private String characterBlacklist;
	private String characterWhitelist;
	private ShutterButton shutterButton;
	private SharedPreferences prefs;
	private OnSharedPreferenceChangeListener listener;
	private ProgressDialog dialog; // for initOcr - language download & unzip
	private ProgressDialog indeterminateDialog; // also for initOcr - init OCR
												// engine
	private boolean isEngineReady;
	private static boolean isFirstLaunch; // True if this is the first time the
											// app is being run

	Handler getHandler() {
		return handler;
	}

	TessBaseAPI getBaseApi() {
		return baseApi;
	}

	CameraManager getCameraManager() {
		return cameraManager;
	}

	@Override
	public void onCreate(Bundle icicle) {
		super.onCreate(icicle);

		checkFirstLaunch();

		if (isFirstLaunch) {
			setDefaultPreferences();
		}

		Window window = getWindow();
		window.addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
		requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
		setContentView(R.layout.capture);
		viewfinderView = (ViewfinderView) findViewById(R.id.viewfinder_view);
		cameraButtonView = findViewById(R.id.camera_button_view);
		resultView = findViewById(R.id.result_view);

		statusViewBottom = (TextView) findViewById(R.id.status_view_bottom);
		registerForContextMenu(statusViewBottom);
		statusViewTop = (TextView) findViewById(R.id.status_view_top);
		registerForContextMenu(statusViewTop);

		handler = null;
		lastResult = null;
		hasSurface = false;
		beepManager = new BeepManager(this);

		// Camera shutter button
		if (DISPLAY_SHUTTER_BUTTON) {
			shutterButton = (ShutterButton) findViewById(R.id.shutter_button);
			shutterButton.setOnShutterButtonListener(this);
		}

		ocrResultView = (TextView) findViewById(R.id.ocr_result_text_view);
		registerForContextMenu(ocrResultView);
		translationView = (TextView) findViewById(R.id.translation_text_view);
		registerForContextMenu(translationView);

		progressView = (View) findViewById(R.id.indeterminate_progress_indicator_view);

		cameraManager = new CameraManager(getApplication());
		viewfinderView.setCameraManager(cameraManager);

		// Set listener to change the size of the viewfinder rectangle.
		viewfinderView.setOnTouchListener(new View.OnTouchListener() {
			int lastX = -1;
			int lastY = -1;

			@Override
			public boolean onTouch(View v, MotionEvent event) {
				switch (event.getAction()) {
				case MotionEvent.ACTION_DOWN:
					lastX = -1;
					lastY = -1;
					return true;
				case MotionEvent.ACTION_MOVE:
					int currentX = (int) event.getX();
					int currentY = (int) event.getY();

					try {
						Rect rect = cameraManager.getFramingRect();

						final int BUFFER = 50;
						final int BIG_BUFFER = 60;
						if (lastX >= 0) {
							// Adjust the size of the viewfinder rectangle.
							// Check if the touch event occurs in the corner
							// areas first, because the regions overlap.
							if (((currentX >= rect.left - BIG_BUFFER && currentX <= rect.left
									+ BIG_BUFFER) || (lastX >= rect.left
									- BIG_BUFFER && lastX <= rect.left
									+ BIG_BUFFER))
									&& ((currentY <= rect.top + BIG_BUFFER && currentY >= rect.top
											- BIG_BUFFER) || (lastY <= rect.top
											+ BIG_BUFFER && lastY >= rect.top
											- BIG_BUFFER))) {
								// Top left corner: adjust both top and left
								// sides
								cameraManager.adjustFramingRect(
										2 * (lastX - currentX),
										2 * (lastY - currentY));
								viewfinderView.removeResultText();
							} else if (((currentX >= rect.right - BIG_BUFFER && currentX <= rect.right
									+ BIG_BUFFER) || (lastX >= rect.right
									- BIG_BUFFER && lastX <= rect.right
									+ BIG_BUFFER))
									&& ((currentY <= rect.top + BIG_BUFFER && currentY >= rect.top
											- BIG_BUFFER) || (lastY <= rect.top
											+ BIG_BUFFER && lastY >= rect.top
											- BIG_BUFFER))) {
								// Top right corner: adjust both top and right
								// sides
								cameraManager.adjustFramingRect(
										2 * (currentX - lastX),
										2 * (lastY - currentY));
								viewfinderView.removeResultText();
							} else if (((currentX >= rect.left - BIG_BUFFER && currentX <= rect.left
									+ BIG_BUFFER) || (lastX >= rect.left
									- BIG_BUFFER && lastX <= rect.left
									+ BIG_BUFFER))
									&& ((currentY <= rect.bottom + BIG_BUFFER && currentY >= rect.bottom
											- BIG_BUFFER) || (lastY <= rect.bottom
											+ BIG_BUFFER && lastY >= rect.bottom
											- BIG_BUFFER))) {
								// Bottom left corner: adjust both bottom and
								// left sides
								cameraManager.adjustFramingRect(
										2 * (lastX - currentX),
										2 * (currentY - lastY));
								viewfinderView.removeResultText();
							} else if (((currentX >= rect.right - BIG_BUFFER && currentX <= rect.right
									+ BIG_BUFFER) || (lastX >= rect.right
									- BIG_BUFFER && lastX <= rect.right
									+ BIG_BUFFER))
									&& ((currentY <= rect.bottom + BIG_BUFFER && currentY >= rect.bottom
											- BIG_BUFFER) || (lastY <= rect.bottom
											+ BIG_BUFFER && lastY >= rect.bottom
											- BIG_BUFFER))) {
								// Bottom right corner: adjust both bottom and
								// right sides
								cameraManager.adjustFramingRect(
										2 * (currentX - lastX),
										2 * (currentY - lastY));
								viewfinderView.removeResultText();
							} else if (((currentX >= rect.left - BUFFER && currentX <= rect.left
									+ BUFFER) || (lastX >= rect.left - BUFFER && lastX <= rect.left
									+ BUFFER))
									&& ((currentY <= rect.bottom && currentY >= rect.top) || (lastY <= rect.bottom && lastY >= rect.top))) {
								// Adjusting left side: event falls within
								// BUFFER pixels of left side, and between top
								// and bottom side limits
								cameraManager.adjustFramingRect(
										2 * (lastX - currentX), 0);
								viewfinderView.removeResultText();
							} else if (((currentX >= rect.right - BUFFER && currentX <= rect.right
									+ BUFFER) || (lastX >= rect.right - BUFFER && lastX <= rect.right
									+ BUFFER))
									&& ((currentY <= rect.bottom && currentY >= rect.top) || (lastY <= rect.bottom && lastY >= rect.top))) {
								// Adjusting right side: event falls within
								// BUFFER pixels of right side, and between top
								// and bottom side limits
								cameraManager.adjustFramingRect(
										2 * (currentX - lastX), 0);
								viewfinderView.removeResultText();
							} else if (((currentY <= rect.top + BUFFER && currentY >= rect.top
									- BUFFER) || (lastY <= rect.top + BUFFER && lastY >= rect.top
									- BUFFER))
									&& ((currentX <= rect.right && currentX >= rect.left) || (lastX <= rect.right && lastX >= rect.left))) {
								// Adjusting top side: event falls within BUFFER
								// pixels of top side, and between left and
								// right side limits
								cameraManager.adjustFramingRect(0,
										2 * (lastY - currentY));
								viewfinderView.removeResultText();
							} else if (((currentY <= rect.bottom + BUFFER && currentY >= rect.bottom
									- BUFFER) || (lastY <= rect.bottom + BUFFER && lastY >= rect.bottom
									- BUFFER))
									&& ((currentX <= rect.right && currentX >= rect.left) || (lastX <= rect.right && lastX >= rect.left))) {
								// Adjusting bottom side: event falls within
								// BUFFER pixels of bottom side, and between
								// left and right side limits
								cameraManager.adjustFramingRect(0,
										2 * (currentY - lastY));
								viewfinderView.removeResultText();
							}
						}
					} catch (NullPointerException e) {
						Log.e(TAG, "Framing rect not available", e);
					}
					v.invalidate();
					lastX = currentX;
					lastY = currentY;
					return true;
				case MotionEvent.ACTION_UP:
					lastX = -1;
					lastY = -1;
					return true;
				}
				return false;
			}
		});

		isEngineReady = false;
	}

	@Override
	protected void onResume() {
		super.onResume();
		resetStatusView();

		String previousSourceLanguageCodeOcr = sourceLanguageCodeOcr;
		int previousOcrEngineMode = ocrEngineMode;

		retrievePreferences();

		// Set up the camera preview surface.
		surfaceView = (SurfaceView) findViewById(R.id.preview_view);
		surfaceHolder = surfaceView.getHolder();
		if (!hasSurface) {
			surfaceHolder.addCallback(this);
			surfaceHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
		}

		// Comment out the following block to test non-OCR functions without an
		// SD card

		// Do OCR engine initialization, if necessary
		boolean doNewInit = (baseApi == null)
				|| !sourceLanguageCodeOcr.equals(previousSourceLanguageCodeOcr)
				|| ocrEngineMode != previousOcrEngineMode;
		if (doNewInit) {
			// Initialize the OCR engine
			File storageDirectory = getStorageDirectory();
			if (storageDirectory != null) {
				initOcrEngine(storageDirectory, sourceLanguageCodeOcr,
						sourceLanguageReadable);
			}
		} else {
			// We already have the engine initialized, so just start the camera.
			resumeOCR();
		}
	}

	/**
	 * @brief Método para iniciar o reiniciar el reconocimiento después de que
	 *        el motor de OCR se ha inicializado, O después de la aplicación
	 *        recupera el foco. Establece el estado y los parámetros de
	 *        configuración relacionados con el motor de OCR, Y las solicitudes
	 *        de inicialización cámara.
	 * @return void
	 */
	void resumeOCR() {
		Log.d(TAG, "resumeOCR()");

		// This method is called when Tesseract has already been successfully
		// initialized, so set
		// isEngineReady = true here.
		isEngineReady = true;

		if (baseApi != null) {
			baseApi.setPageSegMode(pageSegmentationMode);
			baseApi.setVariable(TessBaseAPI.VAR_CHAR_BLACKLIST,
					characterBlacklist);
			baseApi.setVariable(TessBaseAPI.VAR_CHAR_WHITELIST,
					characterWhitelist);
		}

		if (hasSurface) {
			// The activity was paused but not stopped, so the surface still
			// exists. Therefore
			// surfaceCreated() won't be called, so init the camera here.
			initCamera(surfaceHolder);
		}
	}

	@Override
	public void surfaceCreated(SurfaceHolder holder) {
		Log.d(TAG, "surfaceCreated()");

		if (holder == null) {
			Log.e(TAG, "surfaceCreated gave us a null surface");
		}

		// Only initialize the camera if the OCR engine is ready to go.
		if (!hasSurface && isEngineReady) {
			Log.d(TAG, "surfaceCreated(): calling initCamera()...");
			initCamera(holder);
		}
		hasSurface = true;
	}

	/**
	 * @brief Inicializa la cámara y se inicia el controlador para comenzar la
	 *        vista previa.
	 * */
	private void initCamera(SurfaceHolder surfaceHolder) {
		Log.d(TAG, "initCamera()");
		if (surfaceHolder == null) {
			throw new IllegalStateException("No SurfaceHolder provided");
		}
		try {

			// Open and initialize the camera
			cameraManager.openDriver(surfaceHolder);

			// Creating the handler starts the preview, which can also throw a
			// RuntimeException.
			handler = new CaptureActivityHandler(this, cameraManager);

		} catch (IOException ioe) {
			showErrorMessage("Error",
					"Could not initialize camera. Please try restarting device.");
		} catch (RuntimeException e) {
			// Barcode Scanner has seen crashes in the wild of this variety:
			// java.?lang.?RuntimeException: Fail to connect to camera service
			showErrorMessage("Error",
					"Could not initialize camera. Please try restarting device.");
		}
	}

	@Override
	protected void onPause() {
		if (handler != null) {
			handler.quitSynchronously();
		}

		// Stop using the camera, to avoid conflicting with other camera-based
		// apps
		cameraManager.closeDriver();

		if (!hasSurface) {
			SurfaceView surfaceView = (SurfaceView) findViewById(R.id.preview_view);
			SurfaceHolder surfaceHolder = surfaceView.getHolder();
			surfaceHolder.removeCallback(this);
		}
		super.onPause();
	}

	void stopHandler() {
		if (handler != null) {
			handler.stop();
		}
	}

	@Override
	protected void onDestroy() {
		if (baseApi != null) {
			baseApi.end();
		}
		super.onDestroy();
	}

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		if (keyCode == KeyEvent.KEYCODE_BACK) {

			// Exit the app if we're not viewing an OCR result.
			if (lastResult == null) {
				setResult(RESULT_CANCELED);
				finish();
				return true;
			} else {
				// Go back to previewing in regular OCR mode.
				resetStatusView();
				if (handler != null) {
					handler.sendEmptyMessage(R.id.restart_preview);
				}
				return true;
			}
		} else if (keyCode == KeyEvent.KEYCODE_CAMERA) {

			handler.hardwareShutterButtonClick();
			return true;

		} else if (keyCode == KeyEvent.KEYCODE_FOCUS) {
			// Only perform autofocus if user is not holding down the button.
			if (event.getRepeatCount() == 0) {
				cameraManager.requestAutoFocus(500L);
			}
			return true;
		}
		return super.onKeyDown(keyCode, event);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);
		menu.add(0, SETTINGS_ID, 0, getString(R.string.menu_conf)).setIcon(
				android.R.drawable.ic_menu_preferences);
		menu.add(0, ABOUT_ID, 0,  getString(R.string.menu_about)).setIcon(
				android.R.drawable.ic_menu_info_details);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		Intent intent;
		switch (item.getItemId()) {
		case SETTINGS_ID: {
			intent = new Intent().setClass(this, PreferencesActivity.class);
			startActivity(intent);
			break;
		}
		case ABOUT_ID: {
			intent = new Intent(this, HelpActivity.class);
			intent.putExtra(HelpActivity.REQUESTED_PAGE_KEY,
					HelpActivity.ABOUT_PAGE);
			startActivity(intent);
			break;
		}
		}
		return super.onOptionsItemSelected(item);
	}

	public void surfaceDestroyed(SurfaceHolder holder) {
		hasSurface = false;
	}

	public void surfaceChanged(SurfaceHolder holder, int format, int width,
			int height) {
	}

	/**
	 * @brief Metodo que establece los valores lingüísticos necesarios código
	 *        para el lenguaje dado OCR.
	 * @param languageCode Recibe el codigo del idioma
	 * @return boolean
	 */
	private boolean setSourceLanguage(String languageCode) {
		sourceLanguageCodeOcr = languageCode;
		sourceLanguageReadable = LanguageCodeHelper.getOcrLanguageName(this,
				languageCode);
		return true;
	}

	/**
	 * @brief Busca el lugar adecuado en la tarjeta SD en la que podemos guardar
	 *        archivos.
	 * @return File
	 */
	private File getStorageDirectory() {
		// Log.d(TAG, "getStorageDirectory(): API level is " +
		// Integer.valueOf(android.os.Build.VERSION.SDK_INT));

		String state = null;
		try {
			state = Environment.getExternalStorageState();
		} catch (RuntimeException e) {
			Log.e(TAG, "Is the SD card visible?", e);
			showErrorMessage("Error",
					"Required external storage (such as an SD card) is unavailable.");
		}

		if (Environment.MEDIA_MOUNTED.equals(Environment
				.getExternalStorageState())) {

			try {
				return getExternalFilesDir(Environment.MEDIA_MOUNTED);
			} catch (NullPointerException e) {
				// We get an error here if the SD card is visible, but full
				Log.e(TAG, "External storage is unavailable");
				showErrorMessage("Error",
						"Required external storage (such as an SD card) is full or unavailable.");
			}

		} else if (Environment.MEDIA_MOUNTED_READ_ONLY.equals(state)) {
			// We can only read the media
			Log.e(TAG, "External storage is read-only");
			showErrorMessage(
					"Error",
					"Required external storage (such as an SD card) is unavailable for data storage.");
		} else {
			// Something else is wrong. It may be one of many other states, but
			// all we need
			// to know is we can neither read nor write
			Log.e(TAG, "External storage is unavailable");
			showErrorMessage("Error",
					"Required external storage (such as an SD card) is unavailable or corrupted.");
		}
		return null;
	}

	/**
	 * @brief Solicitudes de inicialización del motor de OCR con los parámetros
	 *        dados.    
	 * @param storageRoot ruta a la ubicación del directorio tessdata utilizar.
	 * @param languageCode de tres letras ISO 639-3 códigos de idioma para OCR.
	 * @param languageName nombre de la lengua para OCR, por ejemplo, "Español".
	 * 
	 */
	private void initOcrEngine(File storageRoot, String languageCode,
			String languageName) {
		isEngineReady = false;

		// Set up the dialog box for the thermometer-style download progress
		// indicator
		if (dialog != null) {
			dialog.dismiss();
		}
		dialog = new ProgressDialog(this);

		// Display the name of the OCR engine we're initializing in the
		// indeterminate progress dialog box
		indeterminateDialog = new ProgressDialog(this);
		indeterminateDialog.setTitle(getString(R.string.dialogo_1));
		indeterminateDialog.setCancelable(false);
		indeterminateDialog.show();

		if (handler != null) {
			handler.quitSynchronously();
		}

		// Start AsyncTask to install language data and init OCR
		baseApi = new TessBaseAPI();
		new OcrInitAsyncTask(this, baseApi, dialog, indeterminateDialog,
				languageCode, languageName)
				.execute(storageRoot.toString());
	}

	/**
	 * @brief Muestra información en relación con el resultado de OCR, y pide
	 *        una traducción si es necesario.
	 * @param ocrResult objeto que representa exitosos resultados del OCR.
	 * @return True si un resultado no nulo fue recibido por OCR.
	 */
	boolean handleOcrDecode(OcrResult ocrResult) {
		lastResult = ocrResult;

		// Test whether the result is null
		if (ocrResult.getText() == null || ocrResult.getText().equals("")) {
			Toast toast = Toast.makeText(this, getString(R.string.dialogo_2),
					Toast.LENGTH_SHORT);
			toast.setGravity(Gravity.TOP, 0, 0);
			toast.show();
			return false;
		}

		// Turn off capture-related UI elements
		shutterButton.setVisibility(View.GONE);
		statusViewBottom.setVisibility(View.GONE);
		statusViewTop.setVisibility(View.GONE);
		cameraButtonView.setVisibility(View.GONE);
		viewfinderView.setVisibility(View.GONE);
		resultView.setVisibility(View.VISIBLE);

		ImageView bitmapImageView = (ImageView) findViewById(R.id.image_view);
		lastBitmap = ocrResult.getBitmap();
		if (lastBitmap == null) {
			bitmapImageView.setImageBitmap(BitmapFactory.decodeResource(
					getResources(), R.drawable.ic_launcher));
		} else {
			bitmapImageView.setImageBitmap(lastBitmap);
		}

		// Display the recognized text
		TextView sourceLanguageTextView = (TextView) findViewById(R.id.source_language_text_view);
		sourceLanguageTextView.setText(sourceLanguageReadable);
		TextView ocrResultTextView = (TextView) findViewById(R.id.ocr_result_text_view);
		ocrResultTextView.setText(ocrResult.getText());

		// Crudely scale betweeen 22 and 32 -- bigger font for shorter text
		int scaledSize = Math.max(22, 32 - ocrResult.getText().length() / 4);
		ocrResultTextView.setTextSize(TypedValue.COMPLEX_UNIT_SP, scaledSize);

		TextView translationLanguageLabelTextView = (TextView) findViewById(R.id.translation_language_label_text_view);
		TextView translationLanguageTextView = (TextView) findViewById(R.id.translation_language_text_view);
		TextView translationTextView = (TextView) findViewById(R.id.translation_text_view);

		translationLanguageLabelTextView.setVisibility(View.GONE);
		translationLanguageTextView.setVisibility(View.GONE);
		translationTextView.setVisibility(View.GONE);
		progressView.setVisibility(View.GONE);
		setProgressBarVisibility(false);

		return true;
	}

	@Override
	public void onCreateContextMenu(ContextMenu menu, View v,
			ContextMenuInfo menuInfo) {
		super.onCreateContextMenu(menu, v, menuInfo);
		if (v.equals(ocrResultView)) {
			menu.add(Menu.NONE, OPTIONS_COPY_RECOGNIZED_TEXT_ID, Menu.NONE,
					getString(R.string.dialogo_5));
			menu.add(Menu.NONE, OPTIONS_SHARE_RECOGNIZED_TEXT_ID, Menu.NONE,
					getString(R.string.dialogo_6));
		} else if (v.equals(translationView)) {
			menu.add(Menu.NONE, OPTIONS_COPY_TRANSLATED_TEXT_ID, Menu.NONE,
					getString(R.string.dialogo_5));
			menu.add(Menu.NONE, OPTIONS_SHARE_TRANSLATED_TEXT_ID, Menu.NONE,
					getString(R.string.dialogo_6));
		}
	}

	  @Override
	  public boolean onContextItemSelected(MenuItem item) {
	    ClipboardManager clipboardManager = (ClipboardManager) getSystemService(Context.CLIPBOARD_SERVICE);
	    switch (item.getItemId()) {

	    case OPTIONS_COPY_RECOGNIZED_TEXT_ID:
	        clipboardManager.setText(ocrResultView.getText());
	      if (clipboardManager.hasText()) {
	        Toast toast = Toast.makeText(this, getString(R.string.dialogo_14), Toast.LENGTH_LONG);
	        toast.setGravity(Gravity.BOTTOM, 0, 0);
	        toast.show();
	      }
	      return true;
	    case OPTIONS_SHARE_RECOGNIZED_TEXT_ID:
	    	Intent shareRecognizedTextIntent = new Intent(android.content.Intent.ACTION_SEND);
	    	shareRecognizedTextIntent.setType("text/plain");
	    	shareRecognizedTextIntent.putExtra(android.content.Intent.EXTRA_TEXT, ocrResultView.getText());
	    	startActivity(Intent.createChooser(shareRecognizedTextIntent,  getString(R.string.dialogo_15)));
	    	return true;
	    case OPTIONS_COPY_TRANSLATED_TEXT_ID:
	        clipboardManager.setText(translationView.getText());
	      if (clipboardManager.hasText()) {
	        Toast toast = Toast.makeText(this, getString(R.string.dialogo_14), Toast.LENGTH_LONG);
	        toast.setGravity(Gravity.BOTTOM, 0, 0);
	        toast.show();
	      }
	      return true;
	    case OPTIONS_SHARE_TRANSLATED_TEXT_ID:
	    	Intent shareTranslatedTextIntent = new Intent(android.content.Intent.ACTION_SEND);
	    	shareTranslatedTextIntent.setType("text/plain");
	    	shareTranslatedTextIntent.putExtra(android.content.Intent.EXTRA_TEXT, translationView.getText());
	    	startActivity(Intent.createChooser(shareTranslatedTextIntent,  getString(R.string.dialogo_15)));
	    	return true;
	    default:
	      return super.onContextItemSelected(item);
	    }
	  }
	
	/**
	 * @brief Reseta los elementos de la vista.
	 */
	private void resetStatusView() {
		resultView.setVisibility(View.GONE);

		viewfinderView.setVisibility(View.VISIBLE);
		cameraButtonView.setVisibility(View.VISIBLE);
		if (DISPLAY_SHUTTER_BUTTON) {
			shutterButton.setVisibility(View.VISIBLE);
		}
		lastResult = null;
		viewfinderView.removeResultText();
	}

	/**
	 * @brief Muestra un mensaje pop-up que muestra el nombre de la lengua de
	 *        partida actual OCR.
	 * */
	void showLanguageName() {
		Toast toast = Toast.makeText(this, "OCR: " + sourceLanguageReadable,
				Toast.LENGTH_LONG);
		toast.setGravity(Gravity.TOP, 0, 0);
		toast.show();
	}

	void setButtonVisibility(boolean visible) {
		if (shutterButton != null && visible == true && DISPLAY_SHUTTER_BUTTON) {
			shutterButton.setVisibility(View.VISIBLE);
		} else if (shutterButton != null) {
			shutterButton.setVisibility(View.GONE);
		}
	}

	/**
	 * @brief Activa/desactiva el botón del obturador para evitar doble clic en
	 *        el botón.
	 * @param clickable hacer clic en True si el botón debe aceptar un clic.
	 */
	void setShutterButtonClickable(boolean clickable) {
		shutterButton.setClickable(clickable);
	}

	/** Request the viewfinder to be invalidated. */
	void drawViewfinder() {
		viewfinderView.drawViewfinder();
	}

	@Override
	public void onShutterButtonClick(ShutterButton b) {
		if (handler != null) {
			handler.shutterButtonClick();
		}
	}

	@Override
	public void onShutterButtonFocus(ShutterButton b, boolean pressed) {
		requestDelayedAutoFocus();
	}

	/**
	 * @brief Solicitudes de enfoque automático después de un retardo de 350 ms.
	 *        Este retardo evita foco cuando el usuario solicita hace clic en el
	 *        botón del obturador sin enfocar.
	 */
	private void requestDelayedAutoFocus() {
		// Wait 350 ms before focusing to avoid interfering with quick button
		// presses when
		// the user just wants to take a picture without focusing.
		cameraManager.requestAutoFocus(350L);
	}

	static boolean getFirstLaunch() {
		return isFirstLaunch;
	}

	/**
	 * @brief Queremos la pantalla de ayuda que se muestra automáticamente la
	 *        primera vez que una nueva versión de la aplicación se ejecuta. La
	 *        forma más sencilla de hacer esto es comprobar android: versionCode
	 *        del manifiesto, y comparar a un valor almacenado como una
	 *        preferencia.
	 * 
	 * @return boolean   
	 */
	private boolean checkFirstLaunch() {
		try {
			PackageInfo info = getPackageManager().getPackageInfo(
					getPackageName(), 0);
			int currentVersion = info.versionCode;
			SharedPreferences prefs = PreferenceManager
					.getDefaultSharedPreferences(this);
			int lastVersion = prefs.getInt(
					PreferencesActivity.KEY_HELP_VERSION_SHOWN, 0);
			if (lastVersion == 0) {
				isFirstLaunch = true;
			} else {
				isFirstLaunch = false;
			}
			if (currentVersion > lastVersion) {

				// Record the last version for which we last displayed the
				// What's New (Help) page
				prefs.edit()
						.putInt(PreferencesActivity.KEY_HELP_VERSION_SHOWN,
								currentVersion).commit();
				Intent intent = new Intent(this, HelpActivity.class);
				intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_WHEN_TASK_RESET);

				// Show the default page on a clean install, and the what's new
				// page on an upgrade.
				String page = lastVersion == 0 ? HelpActivity.DEFAULT_PAGE
						: HelpActivity.WHATS_NEW_PAGE;
				intent.putExtra(HelpActivity.REQUESTED_PAGE_KEY, page);
				startActivity(intent);
				return true;
			}
		} catch (PackageManager.NameNotFoundException e) {
			Log.w(TAG, e);
		}
		return false;
	}

	/**
	 * @brief Obtiene los valores de preferencias compartidas y establece los
	 *        miembros de datos correspondientes a esta actividad.
	 */
	private void retrievePreferences() {
		prefs = PreferenceManager.getDefaultSharedPreferences(this);

		// Retrieve from preferences, and set in this Activity, the language
		// preferences
		PreferenceManager.setDefaultValues(this, R.xml.preferences, false);
		setSourceLanguage(prefs.getString(
				PreferencesActivity.KEY_SOURCE_LANGUAGE_PREFERENCE,
				CaptureActivityForOcr.DEFAULT_SOURCE_LANGUAGE_CODE));

		// Retrieve from preferences, and set in this Activity, the page
		// segmentation mode preference
		String[] pageSegmentationModes = getResources().getStringArray(
				R.array.pagesegmentationmodes);
		String pageSegmentationModeName = prefs.getString(
				PreferencesActivity.KEY_PAGE_SEGMENTATION_MODE,
				pageSegmentationModes[0]);
		if (pageSegmentationModeName.equals(pageSegmentationModes[0])) {
			pageSegmentationMode = TessBaseAPI.PageSegMode.PSM_AUTO_OSD;
		} else if (pageSegmentationModeName.equals(pageSegmentationModes[1])) {
			pageSegmentationMode = TessBaseAPI.PageSegMode.PSM_AUTO;
		} else if (pageSegmentationModeName.equals(pageSegmentationModes[2])) {
			pageSegmentationMode = TessBaseAPI.PageSegMode.PSM_SINGLE_BLOCK;
		} else if (pageSegmentationModeName.equals(pageSegmentationModes[3])) {
			pageSegmentationMode = TessBaseAPI.PageSegMode.PSM_SINGLE_CHAR;
		} else if (pageSegmentationModeName.equals(pageSegmentationModes[4])) {
			pageSegmentationMode = TessBaseAPI.PageSegMode.PSM_SINGLE_COLUMN;
		} else if (pageSegmentationModeName.equals(pageSegmentationModes[5])) {
			pageSegmentationMode = TessBaseAPI.PageSegMode.PSM_SINGLE_LINE;
		} else if (pageSegmentationModeName.equals(pageSegmentationModes[6])) {
			pageSegmentationMode = TessBaseAPI.PageSegMode.PSM_SINGLE_WORD;
		} else if (pageSegmentationModeName.equals(pageSegmentationModes[7])) {
			pageSegmentationMode = TessBaseAPI.PageSegMode.PSM_SINGLE_BLOCK_VERT_TEXT;
		}

		// Retrieve from preferences, and set in this Activity, the character
		// blacklist and whitelist
		characterBlacklist = OcrCharacterHelper.getBlacklist(prefs,
				sourceLanguageCodeOcr);
		characterWhitelist = OcrCharacterHelper.getWhitelist(prefs,
				sourceLanguageCodeOcr);

		prefs.registerOnSharedPreferenceChangeListener(listener);

		beepManager.updatePrefs();
	}

	/**
	 * @brief Establece los valores por defecto de las preferencias. Para llamar
	 *        la primera vez que se ejecuta esta aplicación.
	 */
	private void setDefaultPreferences() {
		prefs = PreferenceManager.getDefaultSharedPreferences(this);

		// Recognition language
		prefs.edit()
				.putString(PreferencesActivity.KEY_SOURCE_LANGUAGE_PREFERENCE,
						CaptureActivityForOcr.DEFAULT_SOURCE_LANGUAGE_CODE).commit();

		// Autofocus
		prefs.edit()
				.putBoolean(PreferencesActivity.KEY_AUTO_FOCUS,
						CaptureActivityForOcr.DEFAULT_TOGGLE_AUTO_FOCUS).commit();

		// Beep
		prefs.edit()
				.putBoolean(PreferencesActivity.KEY_PLAY_BEEP,
						CaptureActivityForOcr.DEFAULT_TOGGLE_BEEP).commit();

		// Character blacklist
		prefs.edit()
				.putString(
						PreferencesActivity.KEY_CHARACTER_BLACKLIST,
						OcrCharacterHelper
								.getDefaultBlacklist(CaptureActivityForOcr.DEFAULT_SOURCE_LANGUAGE_CODE))
				.commit();

		// Character whitelist
		prefs.edit()
				.putString(
						PreferencesActivity.KEY_CHARACTER_WHITELIST,
						OcrCharacterHelper
								.getDefaultWhitelist(CaptureActivityForOcr.DEFAULT_SOURCE_LANGUAGE_CODE))
				.commit();

		// Page segmentation mode
		prefs.edit()
				.putString(PreferencesActivity.KEY_PAGE_SEGMENTATION_MODE,
						CaptureActivityForOcr.DEFAULT_PAGE_SEGMENTATION_MODE)
				.commit();

		// Light
		prefs.edit()
				.putBoolean(PreferencesActivity.KEY_TOGGLE_LIGHT,
						CaptureActivityForOcr.DEFAULT_TOGGLE_LIGHT).commit();
	}

	void displayProgressDialog() {
		// Set up the indeterminate progress dialog box
		indeterminateDialog = new ProgressDialog(this);
		indeterminateDialog.setTitle(getString(R.string.dialogo_1));
		String ocrEngineModeName = "Tesseract";

			indeterminateDialog.setMessage(getString(R.string.dialogo_4)
					+ " " + ocrEngineModeName + "...");
			
		indeterminateDialog.setCancelable(false);
		indeterminateDialog.show();
	}

	ProgressDialog getProgressDialog() {
		return indeterminateDialog;
	}

	/**
	 * @brief Muestra un mensaje de error cuadro de diálogo para el usuario en
	 *        el subproceso de interfaz de usuario.
	 * 
	 * @param title El título del título para el cuadro de diálogo
	 * @param message El mensaje del mensaje de error que se mostrará
	 */
	void showErrorMessage(String title, String message) {
		new AlertDialog.Builder(this).setTitle(title).setMessage(message)
				.setOnCancelListener(new FinishListener(this))
				.setPositiveButton(getString(R.string.boton_done), new FinishListener(this)).show();
	}
}
