/*
 * Copyright (C) 2008 ZXing authors
 * Copyright 2011 Robert Theis
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file    CaptureActivityHandler.java
 * @author  Jaime Navarro Santapau
 * @brief   Contiene los métodos y atributos de la clase CaptureActivityHandler
 * @date    Última modificación Diciembre 2012
 * 
 */

package edu.uoc.ocr;

import edu.uoc.ocr.R;
import edu.uoc.ocr.CaptureActivityForOcr;
import edu.uoc.ocr.OcrResult;
import edu.uoc.ocr.camera.CameraManager;

import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.Gravity;
import android.widget.Toast;

/**
 * @class CaptureActivityHandler
 * @brief Esta clase se encarga de gestionar los diferentes estados y
 *        situaciones que comprende el proceso de captura de imágenes.
 */
final class CaptureActivityHandler extends Handler {

	private static final String TAG = CaptureActivityHandler.class
			.getSimpleName();

	private final CaptureActivityForOcr activity;
	private final DecodeThread decodeThread;
	private static State state;
	private final CameraManager cameraManager;

	private enum State {
		PREVIEW, PREVIEW_PAUSED, SUCCESS, DONE
	}

	CaptureActivityHandler(CaptureActivityForOcr activity, CameraManager cameraManager) {
		this.activity = activity;
		this.cameraManager = cameraManager;

		// Start ourselves capturing previews (and decoding if using continuous
		// recognition mode).
		cameraManager.startPreview();

		decodeThread = new DecodeThread(activity);
		decodeThread.start();

		state = State.SUCCESS;

		// Show the shutter and torch buttons
		activity.setButtonVisibility(true);

		restartOcrPreview();

	}

	@Override
	public void handleMessage(Message message) {

		switch (message.what) {
		case R.id.restart_preview:
			restartOcrPreview();
			break;

		case R.id.ocr_decode_succeeded:
			state = State.SUCCESS;
			activity.setShutterButtonClickable(true);
			activity.handleOcrDecode((OcrResult) message.obj);
			break;
		case R.id.ocr_decode_failed:
			state = State.PREVIEW;
			activity.setShutterButtonClickable(true);
			Toast toast = Toast.makeText(activity.getBaseContext(),
					activity.getBaseContext().getString(R.string.dialogo_2),
					Toast.LENGTH_SHORT);
			toast.setGravity(Gravity.TOP, 0, 0);
			toast.show();
			break;
		}
	}

	void stop() {
		// TODO See if this should be done by sending a quit message to
		// decodeHandler as is done
		// below in quitSynchronously().
		Log.d(TAG, "Setting state to CONTINUOUS_PAUSED.");
		removeMessages(R.id.ocr_decode);
	}


	void quitSynchronously() {
		state = State.DONE;
		if (cameraManager != null) {
			cameraManager.stopPreview();
		}
		// Message quit = Message.obtain(decodeThread.getHandler(), R.id.quit);
		try {
			// quit.sendToTarget(); // This always gives
			// "sending message to a Handler on a dead thread"

			// Wait at most half a second; should be enough time, and onPause()
			// will timeout quickly
			decodeThread.join(500L);
		} catch (InterruptedException e) {
			Log.w(TAG, "Caught InterruptedException in quitSyncronously()", e);
			// continue
		} catch (RuntimeException e) {
			Log.w(TAG, "Caught RuntimeException in quitSyncronously()", e);
			// continue
		} catch (Exception e) {
			Log.w(TAG, "Caught unknown Exception in quitSynchronously()", e);
		}

		// Be absolutely sure we don't send any queued up messages
		removeMessages(R.id.ocr_decode);

	}

	/**
	 * @brief Inicia la vista previa, pero no inicia el procesado OCR hasta que el
	 *        usuario pulsa el botón del obturador.
	 */
	private void restartOcrPreview() {
		// Display the shutter and torch buttons
		activity.setButtonVisibility(true);

		if (state == State.SUCCESS) {
			state = State.PREVIEW;

			// Draw the viewfinder.
			activity.drawViewfinder();
		}
	}


	/**
	 * @brief Solicitar OCR en el cuadro de vista previa actual.
	 */
	private void ocrDecode() {
		state = State.PREVIEW_PAUSED;
		cameraManager.requestOcrDecode(decodeThread.getHandler(),R.id.ocr_decode);
	}

	/**
	 * @brief Solicitar OCR cuando el disparador se pulsa hardware.
	 */
	void hardwareShutterButtonClick() {
		// Ensure that we're not in continuous recognition mode
		if (state == State.PREVIEW) {
			ocrDecode();
		}
	}

	/**
	 * @brief Solicitar OCR cuando el botón del obturador en pantalla se hace
	 *        clic.
	 */
	void shutterButtonClick() {
		// Disable further clicks on this button until OCR request is finished
		activity.setShutterButtonClickable(false);
		ocrDecode();
	}

}
