/*
 * Copyright 2011 Robert Theis
 * Copyright 2012 Jaime Navarro Santapau
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file    OcrInitAsyncTask.java
 * @author  Jaime Navarro Santapau
 * @brief   Contiene los métodos y atributos de la clase OcrInitAsyncTask
 * @date    Última modificación Diciembre 2012
 * 
 */
package edu.uoc.ocr;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.RandomAccessFile;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.zip.GZIPInputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import org.xeustechnologies.jtar.TarEntry;
import org.xeustechnologies.jtar.TarInputStream;

import com.googlecode.tesseract.android.TessBaseAPI;

import android.app.ProgressDialog;
import android.content.Context;
import android.os.AsyncTask;
import android.util.Log;

/**
 * @class OcrInitAsyncTask
 * @brief Instala los datos lingüísticos necesarios para el motor OCR, e inicializa el
 *        motor de OCR utilizando un subproceso en segundo plano.
 * 
 */
final class OcrInitAsyncTask extends AsyncTask<String, String, Boolean> {
	private static final String TAG = OcrInitAsyncTask.class.getSimpleName();

	private CaptureActivityForOcr activity;
	private Context context;
	private TessBaseAPI baseApi;
	private ProgressDialog dialog;
	private ProgressDialog indeterminateDialog;
	private final String languageCode;
	private String languageName;

	/**
	 * @brief AsyncTask para descargar de forma asíncrona de datos e inicializar
	 *        Tesseract.
	 * 
	 * @param activity
	 *            La actividad de llamada
	 * @param baseApi
	 *            API para el motor de OCR
	 * @param dialog
	 *            Cuadro de diálogo con termómetro indicador de progreso
	 * @param indeterminateDialog
	 *            Cuadro de diálogo con indicador de progreso indeterminado
	 * @param languageCode
	 *            ISO 639-2 OCR código de idioma
	 * @param languageName
	 *            Nombre del idioma de OCR, por ejemplo, "Inglés"
	 */
	OcrInitAsyncTask(CaptureActivityForOcr activity, TessBaseAPI baseApi,
			ProgressDialog dialog, ProgressDialog indeterminateDialog,
			String languageCode, String languageName) {
		this.activity = activity;
		this.context = activity.getBaseContext();
		this.baseApi = baseApi;
		this.dialog = dialog;
		this.indeterminateDialog = indeterminateDialog;
		this.languageCode = languageCode;
		this.languageName = languageName;
	}

	@Override
	protected void onPreExecute() {
		super.onPreExecute();
		dialog.setTitle(context.getString(R.string.dialogo_1));
		dialog.setMessage(context.getString(R.string.dialogo_12));
		dialog.setIndeterminate(false);
		dialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
		dialog.setCancelable(false);
		dialog.show();
		activity.setButtonVisibility(false);
	}

	/**
	 * @brief En subproceso en segundo plano, realizar la configuración
	 *        necesaria, y la inicialización petición del motor OCR.
	 * 
	 * @param params
	 *            [0] Nombre de ruta del directorio para el almacenamiento de
	 *            datos del lenguaje de archivos a la tarjeta SD
	 */
	protected Boolean doInBackground(String... params) {
		// Check whether we need Cube data or Tesseract data.
		// Example Cube data filename: "tesseract-ocr-3.01.eng.tar"
		// Example Tesseract data filename: "eng.traineddata"
		String destinationFilenameBase = languageCode + ".traineddata";

		// Check for, and create if necessary, folder to hold model data
		String destinationDirBase = params[0]; // The storage directory, minus
												// the
												// "tessdata" subdirectory
		File tessdataDir = new File(destinationDirBase + File.separator
				+ "tessdata");
		if (!tessdataDir.exists() && !tessdataDir.mkdirs()) {
			Log.e(TAG, "Couldn't make directory " + tessdataDir);
			return false;
		}

		// Create a reference to the file to save the download in
		File downloadFile = new File(tessdataDir, destinationFilenameBase);

		// Check if an incomplete download is present. If a *.download file is
		// there, delete it and
		// any (possibly half-unzipped) Tesseract and Cube data files that may
		// be there.
		File incomplete = new File(tessdataDir, destinationFilenameBase
				+ ".download");
		File tesseractTestFile = new File(tessdataDir, languageCode
				+ ".traineddata");
		if (incomplete.exists()) {
			incomplete.delete();
			if (tesseractTestFile.exists()) {
				tesseractTestFile.delete();
			}
		}

		// If language data files are not present, install them
		boolean installSuccess = false;
		if (!tesseractTestFile.exists()) {
			Log.d(TAG, "Language data for " + languageCode + " not found in "
					+ tessdataDir.toString());

			// Check assets for language data to install. If not present,
			// download from Internet
			try {
				Log.d(TAG, "Checking for language data ("
						+ destinationFilenameBase
						+ ".zip) in application assets...");
				// Check for a file like "eng.traineddata.zip" or
				// "tesseract-ocr-3.01.eng.tar.zip"
				installSuccess = installFromAssets(destinationFilenameBase
						+ ".zip", tessdataDir, downloadFile);
			} catch (IOException e) {
				Log.e(TAG, "IOException", e);
			} catch (Exception e) {
				Log.e(TAG, "Got exception", e);
			}

			if (!installSuccess) {
				// File was not packaged in assets, so download it
				Log.d(TAG, "Downloading " + destinationFilenameBase + ".gz...");
				try {
					installSuccess = downloadFile(destinationFilenameBase,
							downloadFile);
					if (!installSuccess) {
						Log.e(TAG, "Download failed");
						return false;
					}
				} catch (IOException e) {
					Log.e(TAG,
							"IOException received in doInBackground. Is a network connection available?");
					return false;
				}
			}

			// If we have a tar file at this point because we downloaded v3.01+
			// data, untar it
			String extension = destinationFilenameBase.substring(
					destinationFilenameBase.lastIndexOf('.'),
					destinationFilenameBase.length());
			if (extension.equals(".tar")) {
				try {
					untar(new File(tessdataDir.toString() + File.separator
							+ destinationFilenameBase), tessdataDir);
					installSuccess = true;
				} catch (IOException e) {
					Log.e(TAG, "Untar failed");
					return false;
				}
			}

		} else {
			Log.d(TAG, "Language data for " + languageCode
					+ " already installed in " + tessdataDir.toString());
			installSuccess = true;
		}

		// If OSD data file is not present, download it
		File osdFile = new File(tessdataDir, CaptureActivityForOcr.OSD_FILENAME_BASE);
		boolean osdInstallSuccess = false;
		if (!osdFile.exists()) {
			// Check assets for language data to install. If not present,
			// download from Internet
			languageName = context.getString(R.string.dialogo_13);
			try {
				// Check for, and delete, partially-downloaded OSD files
				String[] badFiles = {
						CaptureActivityForOcr.OSD_FILENAME + ".gz.download",
						CaptureActivityForOcr.OSD_FILENAME + ".gz",
						CaptureActivityForOcr.OSD_FILENAME };
				for (String filename : badFiles) {
					File file = new File(tessdataDir, filename);
					if (file.exists()) {
						file.delete();
					}
				}

				Log.d(TAG, "Checking for OSD data ("
						+ CaptureActivityForOcr.OSD_FILENAME_BASE
						+ ".zip) in application assets...");
				// Check for "osd.traineddata.zip"
				osdInstallSuccess = installFromAssets(
						CaptureActivityForOcr.OSD_FILENAME_BASE + ".zip",
						tessdataDir, new File(CaptureActivityForOcr.OSD_FILENAME));
			} catch (IOException e) {
				Log.e(TAG, "IOException", e);
			} catch (Exception e) {
				Log.e(TAG, "Got exception", e);
			}

			if (!osdInstallSuccess) {
				// File was not packaged in assets, so download it
				Log.d(TAG, "Downloading " + CaptureActivityForOcr.OSD_FILENAME
						+ ".gz...");
				try {
					osdInstallSuccess = downloadFile(
							CaptureActivityForOcr.OSD_FILENAME, new File(tessdataDir,
									CaptureActivityForOcr.OSD_FILENAME));
					if (!osdInstallSuccess) {
						Log.e(TAG, "Download failed");
						return false;
					}
				} catch (IOException e) {
					Log.e(TAG,
							"IOException received in doInBackground. Is a network connection available?");
					return false;
				}
			}

			// Untar the OSD tar file
			try {
				untar(new File(tessdataDir.toString() + File.separator
						+ CaptureActivityForOcr.OSD_FILENAME), tessdataDir);
			} catch (IOException e) {
				Log.e(TAG, "Untar failed");
				return false;
			}

		} else {
			Log.d(TAG, "OSD file already present in " + tessdataDir.toString());
			osdInstallSuccess = true;
		}

		// Dismiss the progress dialog box, revealing the indeterminate dialog
		// box behind it
		try {
			dialog.dismiss();
		} catch (IllegalArgumentException e) {
			// Catch "View not attached to window manager" error, and continue
		}

		// Initialize the OCR engine
		if (baseApi.init(destinationDirBase + File.separator, languageCode)) {
			return installSuccess && osdInstallSuccess;
		}
		return false;
	}

	/**
	 * @brief Descarga un archivo desde el sitio especificado por DOWNLOAD_BASE,
	 *        y gunzip al destino dado.
	 * 
	 * @param sourceFilenameBase
	 *            Nombre del archivo a descargar, sin el necesario ". Gz"
	 *            extensión
	 * @param destinationFile
	 *            Nombre de archivo para guardar los datos sin comprimir,
	 *            incluyendo la ruta
	 * @return True Si descarga y descompresión tienen éxito
	 * @throws IOException
	 */
	private boolean downloadFile(String sourceFilenameBase, File destinationFile)
			throws IOException {
		try {
			return downloadGzippedFileHttp(
					new URL(CaptureActivityForOcr.DOWNLOAD_BASE + sourceFilenameBase
							+ ".gz"), destinationFile);
		} catch (MalformedURLException e) {
			throw new IllegalArgumentException("Bad URL string.");
		}
	}

	/**
	 * @brief Descargar un archivo comprimido con gzip usando un
	 *        HttpURLConnection, y gunzip al destino dado.
	 * 
	 * @param url
	 *            URL para descargar desde
	 * @param destinationFile
	 *            Archivo para guardar el archivo descargado como, incluyendo la
	 *            ruta
	 * @return True si la respuesta recibida, destinationFile abrió y
	 *         descomprimir con éxito
	 * @throws IOException
	 */
	private boolean downloadGzippedFileHttp(URL url, File destinationFile)
			throws IOException {
		// Send an HTTP GET request for the file
		Log.d(TAG, "Sending GET request to " + url + "...");
		publishProgress(context.getString(R.string.dialogo_3) + " " + languageName + "...", "0");
		HttpURLConnection urlConnection = null;
		urlConnection = (HttpURLConnection) url.openConnection();
		urlConnection.setAllowUserInteraction(false);
		urlConnection.setInstanceFollowRedirects(true);
		urlConnection.setRequestMethod("GET");
		urlConnection.connect();
		if (urlConnection.getResponseCode() != HttpURLConnection.HTTP_OK) {
			Log.e(TAG, "Did not get HTTP_OK response.");
			Log.e(TAG, "Response code: " + urlConnection.getResponseCode());
			Log.e(TAG, "Response message: "
					+ urlConnection.getResponseMessage().toString());
			return false;
		}
		int fileSize = urlConnection.getContentLength();
		InputStream inputStream = urlConnection.getInputStream();
		File tempFile = new File(destinationFile.toString() + ".gz.download");

		// Stream the file contents to a local file temporarily
		Log.d(TAG, "Streaming download to " + destinationFile.toString()
				+ ".gz.download...");
		final int BUFFER = 8192;
		FileOutputStream fileOutputStream = null;
		Integer percentComplete;
		int percentCompleteLast = 0;
		try {
			fileOutputStream = new FileOutputStream(tempFile);
		} catch (FileNotFoundException e) {
			Log.e(TAG, "Exception received when opening FileOutputStream.", e);
		}
		int downloaded = 0;
		byte[] buffer = new byte[BUFFER];
		int bufferLength = 0;
		while ((bufferLength = inputStream.read(buffer, 0, BUFFER)) > 0) {
			fileOutputStream.write(buffer, 0, bufferLength);
			downloaded += bufferLength;
			percentComplete = (int) ((downloaded / (float) fileSize) * 100);
			if (percentComplete > percentCompleteLast) {
				publishProgress(context.getString(R.string.dialogo_3) + " " + languageName
						+ "...", percentComplete.toString());
				percentCompleteLast = percentComplete;
			}
		}
		fileOutputStream.close();
		if (urlConnection != null) {
			urlConnection.disconnect();
		}

		// Uncompress the downloaded temporary file into place, and remove the
		// temporary file
		try {
			Log.d(TAG, "Unzipping...");
			gunzip(tempFile,
					new File(tempFile.toString().replace(".gz.download", "")));
			return true;
		} catch (FileNotFoundException e) {
			Log.e(TAG, "File not available for unzipping.");
		} catch (IOException e) {
			Log.e(TAG, "Problem unzipping file.");
		}
		return false;
	}

	/**
	 * @brief Descomprime el archivo comprimido con gzip dado al destino dado, y
	 *        elimina el archivo comprimido con gzip.
	 * 
	 * @param zippedFile
	 *            El archivo comprimido con gzip
	 * @param outFilePath
	 *            Archivo para descomprimirlo, incluyendo la ruta
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	private void gunzip(File zippedFile, File outFilePath)
			throws FileNotFoundException, IOException {
		int uncompressedFileSize = getGzipSizeUncompressed(zippedFile);
		Integer percentComplete;
		int percentCompleteLast = 0;
		int unzippedBytes = 0;
		final Integer progressMin = 0;
		int progressMax = 100 - progressMin;
		publishProgress(context.getString(R.string.dialogo_7) + " " + languageName + "...",
				progressMin.toString());

		// If the file is a tar file, just show progress to 50%
		String extension = zippedFile.toString().substring(
				zippedFile.toString().length() - 16);
		if (extension.equals(".tar.gz.download")) {
			progressMax = 50;
		}
		GZIPInputStream gzipInputStream = new GZIPInputStream(
				new BufferedInputStream(new FileInputStream(zippedFile)));
		OutputStream outputStream = new FileOutputStream(outFilePath);
		BufferedOutputStream bufferedOutputStream = new BufferedOutputStream(
				outputStream);

		final int BUFFER = 8192;
		byte[] data = new byte[BUFFER];
		int len;
		while ((len = gzipInputStream.read(data, 0, BUFFER)) > 0) {
			bufferedOutputStream.write(data, 0, len);
			unzippedBytes += len;
			percentComplete = (int) ((unzippedBytes / (float) uncompressedFileSize) * progressMax)
					+ progressMin;

			if (percentComplete > percentCompleteLast) {
				publishProgress(context.getString(R.string.dialogo_7) + " " + languageName
						+ "...", percentComplete.toString());
				percentCompleteLast = percentComplete;
			}
		}
		gzipInputStream.close();
		bufferedOutputStream.flush();
		bufferedOutputStream.close();

		if (zippedFile.exists()) {
			zippedFile.delete();
		}
	}

	/**
	 * @brief Devuelve el tamaño sin comprimir de un archivo comprimido con
	 *        gzip.
	 * 
	 * @param file
	 *            Archivo comprimido con gzip para obtener el tamaño de
	 * @return Tamaño sin comprimir, en bytes
	 * @throws IOException
	 */
	private int getGzipSizeUncompressed(File zipFile) throws IOException {
		RandomAccessFile raf = new RandomAccessFile(zipFile, "r");
		raf.seek(raf.length() - 4);
		int b4 = raf.read();
		int b3 = raf.read();
		int b2 = raf.read();
		int b1 = raf.read();
		raf.close();
		return (b1 << 24) | (b2 << 16) + (b3 << 8) + b4;
	}

	/**
	 * @brief Desempaquetar el contenido de un archivo tar en el directorio
	 *        dado, ignorando el nombre de ruta relativa en el archivo tar y
	 *        eliminar el archivo tar.
	 * 
	 *        Usando jtar: http://code.google.com/p/jtar/
	 * 
	 * @param tarFile
	 *            El archivo tar para desempaquetar
	 * @param destinationDir
	 *            El directorio para untar en
	 * @throws IOException
	 */
	private void untar(File tarFile, File destinationDir) throws IOException {
		Log.d(TAG, "Untarring...");
		final int uncompressedSize = getTarSizeUncompressed(tarFile);
		Integer percentComplete;
		int percentCompleteLast = 0;
		int unzippedBytes = 0;
		final Integer progressMin = 50;
		final int progressMax = 100 - progressMin;
		publishProgress(context.getString(R.string.dialogo_7) + " " + languageName + "...",
				progressMin.toString());

		// Extract all the files
		TarInputStream tarInputStream = new TarInputStream(
				new BufferedInputStream(new FileInputStream(tarFile)));
		TarEntry entry;
		while ((entry = tarInputStream.getNextEntry()) != null) {
			int len;
			final int BUFFER = 8192;
			byte data[] = new byte[BUFFER];
			String pathName = entry.getName();
			String fileName = pathName.substring(pathName.lastIndexOf('/'),
					pathName.length());
			OutputStream outputStream = new FileOutputStream(destinationDir
					+ fileName);
			BufferedOutputStream bufferedOutputStream = new BufferedOutputStream(
					outputStream);

			Log.d(TAG, "Writing " + fileName.substring(1, fileName.length())
					+ "...");
			while ((len = tarInputStream.read(data, 0, BUFFER)) != -1) {
				bufferedOutputStream.write(data, 0, len);
				unzippedBytes += len;
				percentComplete = (int) ((unzippedBytes / (float) uncompressedSize) * progressMax)
						+ progressMin;
				if (percentComplete > percentCompleteLast) {
					publishProgress(context.getString(R.string.dialogo_7) + " "
							+ languageName + "...", percentComplete.toString());
					percentCompleteLast = percentComplete;
				}
			}
			bufferedOutputStream.flush();
			bufferedOutputStream.close();
		}
		tarInputStream.close();

		if (tarFile.exists()) {
			tarFile.delete();
		}
	}

	/**
	 * @brief Devuelve el tamaño sin comprimir de un archivo Tar.
	 * 
	 * @param tarFile
	 *            El archivo alquitranado
	 * @return Size cuando no comprimidos, en bytes
	 * @throws IOException
	 */
	private int getTarSizeUncompressed(File tarFile) throws IOException {
		int size = 0;
		TarInputStream tis = new TarInputStream(new BufferedInputStream(
				new FileInputStream(tarFile)));
		TarEntry entry;
		while ((entry = tis.getNextEntry()) != null) {
			if (!entry.isDirectory()) {
				size += entry.getSize();
			}
		}
		return size;
	}

	/**
	 * @brief Instalación de un archivo de activos de aplicaciones para
	 *        dispositivos de almacenamiento externo.
	 * 
	 * @param sourceFilename
	 *            Archivo de activos para instalar
	 * @param modelRoot
	 *            Directorio en la tarjeta SD para instalar el archivo de
	 * @param destinationFile
	 *            Nombre de archivo de destino, con exclusión de ruta
	 * @return True Si installZipFromAssets devuelve true
	 * @throws IOException
	 */
	private boolean installFromAssets(String sourceFilename, File modelRoot,
			File destinationFile) throws IOException {
		String extension = sourceFilename.substring(
				sourceFilename.lastIndexOf('.'), sourceFilename.length());
		try {
			if (extension.equals(".zip")) {
				return installZipFromAssets(sourceFilename, modelRoot,
						destinationFile);
			} else {
				throw new IllegalArgumentException("Extension " + extension
						+ " is unsupported.");
			}
		} catch (FileNotFoundException e) {
			Log.d(TAG, "Language not packaged in application assets.");
		}
		return false;
	}

	/**
	 * @brief Descomprimir el archivo zip dado, ubicado en el activo de la
	 *        aplicación, en el archivo de destino determinado.
	 * 
	 * @param sourceFilename
	 *            Nombre del archivo en activos
	 * @param destinationDir
	 *            Directorio para guardar el archivo de destino en
	 * @param destinationFile
	 *            Archivo para descomprimir en, excluyendo camino
	 * @return boolean
	 * @throws IOException
	 * @throws FileNotFoundException
	 */
	private boolean installZipFromAssets(String sourceFilename,
			File destinationDir, File destinationFile) throws IOException,
			FileNotFoundException {
		// Attempt to open the zip archive
		publishProgress(context.getString(R.string.dialogo_7) + " " + languageName + "...",
				"0");
		ZipInputStream inputStream = new ZipInputStream(context.getAssets()
				.open(sourceFilename));

		// Loop through all the files and folders in the zip archive (but there
		// should just be one)
		for (ZipEntry entry = inputStream.getNextEntry(); entry != null; entry = inputStream
				.getNextEntry()) {
			destinationFile = new File(destinationDir, entry.getName());

			if (entry.isDirectory()) {
				destinationFile.mkdirs();
			} else {
				// Note getSize() returns -1 when the zipfile does not have the
				// size set
				long zippedFileSize = entry.getSize();

				// Create a file output stream
				FileOutputStream outputStream = new FileOutputStream(
						destinationFile);
				final int BUFFER = 8192;

				// Buffer the output to the file
				BufferedOutputStream bufferedOutputStream = new BufferedOutputStream(
						outputStream, BUFFER);
				int unzippedSize = 0;

				// Write the contents
				int count = 0;
				Integer percentComplete = 0;
				Integer percentCompleteLast = 0;
				byte[] data = new byte[BUFFER];
				while ((count = inputStream.read(data, 0, BUFFER)) != -1) {
					bufferedOutputStream.write(data, 0, count);
					unzippedSize += count;
					percentComplete = (int) ((unzippedSize / (long) zippedFileSize) * 100);
					if (percentComplete > percentCompleteLast) {
						publishProgress(context.getString(R.string.dialogo_7) + " "
								+ languageName + "...",
								percentComplete.toString(), "0");
						percentCompleteLast = percentComplete;
					}
				}
				bufferedOutputStream.close();
			}
			inputStream.closeEntry();
		}
		inputStream.close();
		return true;
	}

	/**
	 * @ brief Actualizar el cuadro de diálogo con los últimos avances
	 * incrementales.
	 * 
	 * @param message
	 *            [0] El texto que se mostrará
	 * @param message
	 *            [1] Valor numérico para el progreso
	 */
	@Override
	protected void onProgressUpdate(String... message) {
		super.onProgressUpdate(message);
		int percentComplete = 0;

		percentComplete = Integer.parseInt(message[1]);
		dialog.setMessage(message[0]);
		dialog.setProgress(percentComplete);
		dialog.show();
	}

	@Override
	protected void onPostExecute(Boolean result) {
		super.onPostExecute(result);

		try {
			indeterminateDialog.dismiss();
		} catch (IllegalArgumentException e) {
			// Catch "View not attached to window manager" error, and continue
		}

		if (result) {
			// Restart recognition
			activity.resumeOCR();
			activity.showLanguageName();
		} else {
			activity.showErrorMessage(
					context.getString(R.string.dialogo_10),
					context.getString(R.string.dialogo_11));
		}
	}
}