/*
 * Copyright 2011 Robert Theis
 * Copyright 2012 Jaime Navarro Santapau
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package edu.uoc.ocr;


/**
 * @file    OcrRecognizeAsyncTask.java
 * @author  Jaime Navarro Santapau
 * @brief   Contiene los métodos y atributos de la clase OcrRecognizeAsyncTask
 * @date    Última modificación Diciembre 2012
 * 
 */
import com.googlecode.leptonica.android.ReadFile;
import com.googlecode.tesseract.android.TessBaseAPI;

import edu.uoc.ocr.R;

import android.graphics.Bitmap;
import android.os.AsyncTask;
import android.os.Handler;
import android.os.Message;
import android.util.Log;


/**
 * @class OcrRecognizeAsyncTask
 * @brief Clase para enviar solicitudes al motor de OCR en un hilo separado,
 *        envía un mensaje de éxito/fracaso, y cierra el cuadro de diálogo de
 *        progreso.
 * 
 */
final class OcrRecognizeAsyncTask extends AsyncTask<Void, Void, Boolean> {

	private CaptureActivityForOcr activity;
	private TessBaseAPI baseApi;
	private byte[] data;
	private int width;
	private int height;
	private OcrResult ocrResult;
	private long timeRequired;

	OcrRecognizeAsyncTask(CaptureActivityForOcr activity, TessBaseAPI baseApi,
			byte[] data, int width, int height) {
		this.activity = activity;
		this.baseApi = baseApi;
		this.data = data;
		this.width = width;
		this.height = height;
	}

	@Override
	protected Boolean doInBackground(Void... arg0) {
		long start = System.currentTimeMillis();
		Bitmap bitmap = activity.getCameraManager()
				.buildLuminanceSource(data, width, height)
				.renderCroppedGreyscaleBitmap();
		String textResult;

		try {
			baseApi.setImage(ReadFile.readBitmap(bitmap));
			textResult = baseApi.getUTF8Text();
			timeRequired = System.currentTimeMillis() - start;

			// Check for failure to recognize text
			if (textResult == null || textResult.equals("")) {
				return false;
			}
			ocrResult = new OcrResult();
			ocrResult.setWordConfidences(baseApi.wordConfidences());
			ocrResult.setMeanConfidence(baseApi.meanConfidence());
			ocrResult.setRegionBoundingBoxes(baseApi.getRegions().getBoxRects());
			ocrResult.setWordBoundingBoxes(baseApi.getWords().getBoxRects());

		} catch (RuntimeException e) {
			Log.e("OcrRecognizeAsyncTask",
					"Caught RuntimeException in request to Tesseract. Setting state to CONTINUOUS_STOPPED.");
			e.printStackTrace();
			try {
				baseApi.clear();
				activity.stopHandler();
			} catch (NullPointerException e1) {
				// Continue
			}
			return false;
		}
		timeRequired = System.currentTimeMillis() - start;
		ocrResult.setBitmap(bitmap);
		ocrResult.setText(textResult);
		ocrResult.setRecognitionTimeRequired(timeRequired);
		return true;
	}

	@Override
	protected void onPostExecute(Boolean result) {
		super.onPostExecute(result);

		Handler handler = activity.getHandler();
		if (handler != null) {
			// Send results for single-shot mode recognition.
			if (result) {
				Message message = Message.obtain(handler,
						R.id.ocr_decode_succeeded, ocrResult);
				message.sendToTarget();
			} else {
				Message message = Message.obtain(handler,
						R.id.ocr_decode_failed, ocrResult);
				message.sendToTarget();
			}
			activity.getProgressDialog().dismiss();
		}
		if (baseApi != null) {
			baseApi.clear();
		}
	}
}
