/*
 * Copyright (C) 2008 ZXing authors
 * Copyright 2011 Robert Theis
 * Copyright 2012 Jaime Navarro Santapau
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file    PreferencesActivity.java
 * @author  Jaime Navarro Santapau
 * @brief   Contiene los métodos y atributos de la clase PreferencesActivity
 * @date    Última modificación Diciembre 2012
 * 
 */
package edu.uoc.ocr;


import java.io.File;

import edu.uoc.ocr.R;
import edu.uoc.ocr.LanguageCodeHelper;

import android.app.AlertDialog;
import android.content.SharedPreferences;
import android.content.SharedPreferences.OnSharedPreferenceChangeListener;
import android.os.Bundle;
import android.os.Environment;
import android.preference.EditTextPreference;
import android.preference.ListPreference;
import android.preference.PreferenceActivity;
import android.preference.PreferenceManager;

/**
 * @class PreferencesActivity
 * @brief Actividad para manejar las preferencias que se guardan en cada sesión
 *        de la aplicación. Muestra una jerarquía de preferencias para el
 *        usuario, organizadas en secciones. Estas preferencias se muestran en
 *        el menú de opciones que aparece cuando el usuario pulsa el botón de
 *        menú.
 */
public class PreferencesActivity extends PreferenceActivity implements
		OnSharedPreferenceChangeListener {

	public static final String KEY_SOURCE_LANGUAGE_PREFERENCE = "sourceLanguageCodeOcrPref";
	public static final String KEY_CHARACTER_BLACKLIST = "preference_character_blacklist";
	public static final String KEY_CHARACTER_WHITELIST = "preference_character_whitelist";
	public static final String KEY_PAGE_SEGMENTATION_MODE = "preference_page_segmentation_mode";

	public static final String KEY_AUTO_FOCUS = "preferences_auto_focus";
	public static final String KEY_TOGGLE_LIGHT = "preference_toggle_light";
	public static final String KEY_PLAY_BEEP = "preferences_play_beep";

	public static final String KEY_HELP_VERSION_SHOWN = "preferences_help_version_shown";
	public static final String KEY_NOT_OUR_RESULTS_SHOWN = "preferences_not_our_results_shown";
	public static final String KEY_REVERSE_IMAGE = "preferences_reverse_image";

	public static final String KEY_VIBRATE = "preferences_vibrate";

	private ListPreference listPreferenceSourceLanguage;

	private EditTextPreference editTextPreferenceCharacterBlacklist;
	private EditTextPreference editTextPreferenceCharacterWhitelist;
	private ListPreference listPreferencePageSegmentationMode;

	private static SharedPreferences sharedPreferences;

	/**
	 * @brief Establezca los valores de preferencia predeterminados.
	 * 
	 * @param Bundle
	 *            savedInstanceState estado de la actividad en curso, como pasó
	 *            por Android
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		addPreferencesFromResource(R.xml.preferences);

		sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);

		listPreferenceSourceLanguage = (ListPreference) getPreferenceScreen()
				.findPreference(KEY_SOURCE_LANGUAGE_PREFERENCE);

		editTextPreferenceCharacterBlacklist = (EditTextPreference) getPreferenceScreen()
				.findPreference(KEY_CHARACTER_BLACKLIST);
		editTextPreferenceCharacterWhitelist = (EditTextPreference) getPreferenceScreen()
				.findPreference(KEY_CHARACTER_WHITELIST);
		listPreferencePageSegmentationMode = (ListPreference) getPreferenceScreen()
				.findPreference(KEY_PAGE_SEGMENTATION_MODE);

	}

	/**
	 * @brief Definición de interfaz para una devolución de llamada que se
	 *        invoca cuando una preferencia compartida se cambia. Establece el
	 *        texto resumido de las preferencias de la aplicación. Los valores
	 *        de resumen de texto muestran los valores actuales de los valores.
	 * @param sharedPreferences
	 *            Android.content.SharedPreferences los que recibieron el cambio
	 * @param key
	 *            a clave de la preferencia que se ha cambiado, añadido o
	 *            eliminado
	 */
	@Override
	public void onSharedPreferenceChanged(SharedPreferences sharedPreferences,
			String key) {

		if (key.equals(KEY_SOURCE_LANGUAGE_PREFERENCE)) {

			File appDir = getExternalFilesDir(Environment.MEDIA_MOUNTED);

			if (LanguageCodeHelper.getOcrLanguageName(
					getBaseContext(),
					sharedPreferences.getString(key,
							CaptureActivityForOcr.DEFAULT_SOURCE_LANGUAGE_CODE))
					.contentEquals(getString(R.string.patron_manual))) {
				File tesseractTestFile = new File(appDir, "tessdata"
						+ File.separator + CaptureActivityForOcr.MANUAL_FILENAME_BASE);
				if (!tesseractTestFile.exists()) {

					new AlertDialog.Builder(this)
							.setTitle(getString(R.string.dialogo_8))
							.setMessage(getString(R.string.dialogo_9))
							.show();

					listPreferenceSourceLanguage.setValueIndex(1);
					listPreferenceSourceLanguage.setSummary("Patrón Español");

				} else {

					// Set the summary text for the source language name
					listPreferenceSourceLanguage
							.setSummary(LanguageCodeHelper.getOcrLanguageName(
									getBaseContext(),
									sharedPreferences
											.getString(
													key,
													CaptureActivityForOcr.DEFAULT_SOURCE_LANGUAGE_CODE)));

					// Retrieve the character blacklist/whitelist for the new
					// language
					String blacklist = OcrCharacterHelper.getBlacklist(
							sharedPreferences,
							listPreferenceSourceLanguage.getValue());
					String whitelist = OcrCharacterHelper.getWhitelist(
							sharedPreferences,
							listPreferenceSourceLanguage.getValue());

					// Save the character blacklist/whitelist to preferences
					sharedPreferences.edit()
							.putString(KEY_CHARACTER_BLACKLIST, blacklist)
							.commit();
					sharedPreferences.edit()
							.putString(KEY_CHARACTER_WHITELIST, whitelist)
							.commit();

					// Set the blacklist/whitelist summary text
					editTextPreferenceCharacterBlacklist.setSummary(blacklist);
					editTextPreferenceCharacterWhitelist.setSummary(whitelist);

				}

			} else {
				// Set the summary text for the source language name
				listPreferenceSourceLanguage
						.setSummary(LanguageCodeHelper.getOcrLanguageName(
								getBaseContext(),
								sharedPreferences
										.getString(
												key,
												CaptureActivityForOcr.DEFAULT_SOURCE_LANGUAGE_CODE)));

				// Retrieve the character blacklist/whitelist for the new
				// language
				String blacklist = OcrCharacterHelper.getBlacklist(
						sharedPreferences,
						listPreferenceSourceLanguage.getValue());
				String whitelist = OcrCharacterHelper.getWhitelist(
						sharedPreferences,
						listPreferenceSourceLanguage.getValue());

				// Save the character blacklist/whitelist to preferences
				sharedPreferences.edit()
						.putString(KEY_CHARACTER_BLACKLIST, blacklist).commit();
				sharedPreferences.edit()
						.putString(KEY_CHARACTER_WHITELIST, whitelist).commit();

				// Set the blacklist/whitelist summary text
				editTextPreferenceCharacterBlacklist.setSummary(blacklist);
				editTextPreferenceCharacterWhitelist.setSummary(whitelist);
			}
		} else if (key.equals(KEY_PAGE_SEGMENTATION_MODE)) {
			listPreferencePageSegmentationMode.setSummary(sharedPreferences
					.getString(key,
							CaptureActivityForOcr.DEFAULT_PAGE_SEGMENTATION_MODE));

		} else if (key.equals(KEY_CHARACTER_BLACKLIST)) {

			// Save a separate, language-specific character blacklist for this
			// language
			OcrCharacterHelper.setBlacklist(sharedPreferences,
					listPreferenceSourceLanguage.getValue(),
					sharedPreferences.getString(key, OcrCharacterHelper
							.getDefaultBlacklist(listPreferenceSourceLanguage
									.getValue())));

			// Set the summary text
			editTextPreferenceCharacterBlacklist.setSummary(sharedPreferences
					.getString(key, OcrCharacterHelper
							.getDefaultBlacklist(listPreferenceSourceLanguage
									.getValue())));

		} else if (key.equals(KEY_CHARACTER_WHITELIST)) {

			// Save a separate, language-specific character blacklist for this
			// language
			OcrCharacterHelper.setWhitelist(sharedPreferences,
					listPreferenceSourceLanguage.getValue(),
					sharedPreferences.getString(key, OcrCharacterHelper
							.getDefaultWhitelist(listPreferenceSourceLanguage
									.getValue())));

			// Set the summary text
			editTextPreferenceCharacterWhitelist.setSummary(sharedPreferences
					.getString(key, OcrCharacterHelper
							.getDefaultWhitelist(listPreferenceSourceLanguage
									.getValue())));

		}

	}

	/**
	 * @brief Establece los valores iniciales de preferencias de texto de
	 *        resumen y registra el OnSharedPreferenceChangeListener.
	 */
	@Override
	protected void onResume() {
		super.onResume();
		// Set up the initial summary values
		listPreferenceSourceLanguage.setSummary(LanguageCodeHelper
				.getOcrLanguageName(getBaseContext(), sharedPreferences
						.getString(KEY_SOURCE_LANGUAGE_PREFERENCE,
								CaptureActivityForOcr.DEFAULT_SOURCE_LANGUAGE_CODE)));
		listPreferencePageSegmentationMode.setSummary(sharedPreferences
				.getString(KEY_PAGE_SEGMENTATION_MODE,
						CaptureActivityForOcr.DEFAULT_PAGE_SEGMENTATION_MODE));
		editTextPreferenceCharacterBlacklist.setSummary(sharedPreferences
				.getString(KEY_CHARACTER_BLACKLIST, OcrCharacterHelper
						.getDefaultBlacklist(listPreferenceSourceLanguage
								.getValue())));
		editTextPreferenceCharacterWhitelist.setSummary(sharedPreferences
				.getString(KEY_CHARACTER_WHITELIST, OcrCharacterHelper
						.getDefaultWhitelist(listPreferenceSourceLanguage
								.getValue())));

		// Set up a listener whenever a key changes
		getPreferenceScreen().getSharedPreferences()
				.registerOnSharedPreferenceChangeListener(this);
	}

	/**
	 * @brief Se le llama cuando la actividad está a punto de perder el foco.
	 *        Anula el registro del OnSharedPreferenceChangeListener.
	 */
	@Override
	protected void onPause() {
		super.onPause();
		getPreferenceScreen().getSharedPreferences()
				.unregisterOnSharedPreferenceChangeListener(this);
	}
}
