/*
 * Copyright (C) 2008 ZXing authors
 * Copyright 2011 Robert Theis
 * Copyright 2012 Jaime Navarro Santapau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file    CameraManager.java
 * @author  Jaime Navarro Santapau
 * @brief   Contiene los métodos y atributos de la clase CameraManager
 * @date    Última modificación Diciembre 2012
 * 
 */
package edu.uoc.ocr.camera;

import android.content.Context;
import android.content.SharedPreferences;
import android.graphics.Point;
import android.graphics.Rect;
import android.hardware.Camera;
import android.os.Handler;
import android.preference.PreferenceManager;
import android.view.SurfaceHolder;
import edu.uoc.ocr.PlanarYUVLuminanceSource;
import edu.uoc.ocr.PreferencesActivity;

import java.io.IOException;

/**
 * @class CameraManager
 * @brief Este clase contiene los servicios de la cámara y espera ser el
 *        única que interactúe con ella. La aplicación encapsula los pasos necesarios
 *        para tomar imágenes de tamaño de vista previa, que se utilizan tanto
 *        para la vista previa y la decodificación.
 */
public final class CameraManager {

	private static final int MIN_FRAME_WIDTH = 50; // originally 240
	private static final int MIN_FRAME_HEIGHT = 20; // originally 240
	private static final int MAX_FRAME_WIDTH = 800; // originally 480
	private static final int MAX_FRAME_HEIGHT = 600; // originally 360

	private final Context context;
	private final CameraConfigurationManager configManager;
	private Camera camera;
	private AutoFocusManager autoFocusManager;
	private Rect framingRect;
	private Rect framingRectInPreview;
	private boolean initialized;
	private boolean previewing;
	private boolean reverseImage;
	private int requestedFramingRectWidth;
	private int requestedFramingRectHeight;

	/**
	 * @brief Cuadros de vista preliminar se entregan aquí, que pasamos a el
	 *        controlador registrado. Asegúrese de desactivar el controlador por
	 *        lo que sólo recibirá un mensaje.
	 */
	private final PreviewCallback previewCallback;

	public CameraManager(Context context) {
		this.context = context;
		this.configManager = new CameraConfigurationManager(context);
		previewCallback = new PreviewCallback(configManager);
	}

	/**
	 * @brief Abre el controlador de la cámara e inicializa los parámetros de
	 *        hardware.
	 * 
	 * @param holder
	 *            La superficie objeto de que la cámara se basará en cuadros de
	 *            vista preliminar.
	 * @throws IOException
	 *             Indica que el controlador de la cámara no se puede abrir.
	 */
	public synchronized void openDriver(SurfaceHolder holder)
			throws IOException {
		Camera theCamera = camera;
		if (theCamera == null) {
			theCamera = Camera.open();
			if (theCamera == null) {
				throw new IOException();
			}
			camera = theCamera;
		}
		camera.setPreviewDisplay(holder);
		if (!initialized) {
			initialized = true;
			configManager.initFromCameraParameters(theCamera);
			if (requestedFramingRectWidth > 0 && requestedFramingRectHeight > 0) {
				adjustFramingRect(requestedFramingRectWidth,
						requestedFramingRectHeight);
				requestedFramingRectWidth = 0;
				requestedFramingRectHeight = 0;
			}
		}
		configManager.setDesiredCameraParameters(theCamera);

		SharedPreferences prefs = PreferenceManager
				.getDefaultSharedPreferences(context);
		reverseImage = prefs.getBoolean(PreferencesActivity.KEY_REVERSE_IMAGE,
				false);
	}

	/**
	 * @brief Cierra el controlador de la cámara si todavía está en uso.
	 */
	public synchronized void closeDriver() {
		if (camera != null) {
			camera.release();
			camera = null;

			// Make sure to clear these each time we close the camera, so that
			// any scanning rect
			// requested by intent is forgotten.
			framingRect = null;
			framingRectInPreview = null;
		}
	}

	/**
	 * @brief Pide al hardware de la cámara para empezar a dibujar cuadros de
	 *        vista preliminar en la pantalla.
	 */
	public synchronized void startPreview() {
		Camera theCamera = camera;
		if (theCamera != null && !previewing) {
			theCamera.startPreview();
			previewing = true;
			autoFocusManager = new AutoFocusManager(context, camera);
		}
	}

	/**
	 * @brief Indica a la cámara para dejar de dibujar cuadros de vista
	 *        preliminar.
	 */
	public synchronized void stopPreview() {
		if (autoFocusManager != null) {
			autoFocusManager.stop();
			autoFocusManager = null;
		}
		if (camera != null && previewing) {
			camera.stopPreview();
			previewCallback.setHandler(null, 0);
			previewing = false;
		}
	}

	/**
	 * @briefUn marco de vista previa solo se devolverá al controlador
	 *          suministrado. Los datos llegarán como byte [] en el campo
	 *          message.obj, con ancho y altura codificada como message.arg1 y
	 *          message.arg2, respectivamente.
	 * 
	 * @param handler
	 *            El controlador para enviar el mensaje.
	 * @param message
	 *            El campo de lo que el mensaje sea enviado.
	 */
	public synchronized void requestOcrDecode(Handler handler, int message) {
		Camera theCamera = camera;
		if (theCamera != null && previewing) {
			previewCallback.setHandler(handler, message);
			theCamera.setOneShotPreviewCallback(previewCallback);
		}
	}

	/**
	 * @brief Pide al hardware de la cámara para realizar un enfoque automático.
	 * @param delay
	 *            Tiempo de retardo para enviar la solicitud
	 */
	public synchronized void requestAutoFocus(long delay) {
		autoFocusManager.start(delay);
	}

	/**
	 * @brief Calcula el encuadre rect que la interfaz de usuario debe dibujar
	 *        para mostrar al usuario dónde colocar el código de barras. Este
	 *        objetivo ayuda con la alineación así como las fuerzas del usuario
	 *        para mantener el dispositivo lo suficientemente lejos para
	 *        garantizar que la imagen estará en foco.
	 * 
	 * @return El rectángulo para dibujar en la pantalla en coordenadas de la
	 *         ventana.
	 */
	public synchronized Rect getFramingRect() {
		if (framingRect == null) {
			if (camera == null) {
				return null;
			}
			Point screenResolution = configManager.getScreenResolution();
			if (screenResolution == null) {
				// Called early, before init even finished
				return null;
			}
			int width = screenResolution.x * 3 / 5;
			if (width < MIN_FRAME_WIDTH) {
				width = MIN_FRAME_WIDTH;
			} else if (width > MAX_FRAME_WIDTH) {
				width = MAX_FRAME_WIDTH;
			}
			int height = screenResolution.y * 1 / 5;
			if (height < MIN_FRAME_HEIGHT) {
				height = MIN_FRAME_HEIGHT;
			} else if (height > MAX_FRAME_HEIGHT) {
				height = MAX_FRAME_HEIGHT;
			}
			int leftOffset = (screenResolution.x - width) / 2;
			int topOffset = (screenResolution.y - height) / 2;
			framingRect = new Rect(leftOffset, topOffset, leftOffset + width,
					topOffset + height);
		}
		return framingRect;
	}

	/**
	 * @brief Al igual que getFramingRect pero las coordenadas están en términos
	 *        del marco de vista previa no, UI / pantalla.
	 */
	public synchronized Rect getFramingRectInPreview() {
		if (framingRectInPreview == null) {
			Rect rect = new Rect(getFramingRect());
			Point cameraResolution = configManager.getCameraResolution();
			Point screenResolution = configManager.getScreenResolution();
			if (cameraResolution == null || screenResolution == null) {
				// Called early, before init even finished
				return null;
			}
			rect.left = rect.left * cameraResolution.x / screenResolution.x;
			rect.right = rect.right * cameraResolution.x / screenResolution.x;
			rect.top = rect.top * cameraResolution.y / screenResolution.y;
			rect.bottom = rect.bottom * cameraResolution.y / screenResolution.y;
			framingRectInPreview = rect;
		}
		return framingRectInPreview;
	}

	/**
	 * @brief Cambia el tamaño de la formulación rect.
	 * 
	 * @param deltaWidth
	 *            Número de píxeles para ajustar el ancho
	 * @param deltaHeight
	 *            Número de píxeles para ajustar la altura
	 */
	public synchronized void adjustFramingRect(int deltaWidth, int deltaHeight) {
		if (initialized) {
			Point screenResolution = configManager.getScreenResolution();

			// Set maximum and minimum sizes
			if ((framingRect.width() + deltaWidth > screenResolution.x - 4)
					|| (framingRect.width() + deltaWidth < 50)) {
				deltaWidth = 0;
			}
			if ((framingRect.height() + deltaHeight > screenResolution.y - 4)
					|| (framingRect.height() + deltaHeight < 50)) {
				deltaHeight = 0;
			}

			int newWidth = framingRect.width() + deltaWidth;
			int newHeight = framingRect.height() + deltaHeight;
			int leftOffset = (screenResolution.x - newWidth) / 2;
			int topOffset = (screenResolution.y - newHeight) / 2;
			framingRect = new Rect(leftOffset, topOffset,
					leftOffset + newWidth, topOffset + newHeight);
			framingRectInPreview = null;
		} else {
			requestedFramingRectWidth = deltaWidth;
			requestedFramingRectHeight = deltaHeight;
		}
	}

	/**
	 * @brief Un método de fábrica para construir el objeto LuminanceSource
	 *        apropiado basado en el formato de los buffers de vista previa,
	 *        como se describe por Camera.Parameters.
	 * 
	 * @param data
	 *            Un cuadro de vista previa.
	 * @param width
	 *            La anchura de la imagen.
	 * @param height
	 *            La altura de la imagen.
	 * @return Un ejemplo PlanarYUVLuminanceSource.
	 */
	public PlanarYUVLuminanceSource buildLuminanceSource(byte[] data,
			int width, int height) {
		Rect rect = getFramingRectInPreview();
		if (rect == null) {
			return null;
		}
		// Go ahead and assume it's YUV rather than die.
		return new PlanarYUVLuminanceSource(data, width, height, rect.left,
				rect.top, rect.width(), rect.height(), reverseImage);
	}

}

